#ifndef XML_NODE_H
#define XML_NODE_H

#include <vector>
#include <string>
#include <iostream>
#include <stdio.h>

class CxmlNode
{
  private:
    class CxmlAttribute
    {
      private:
        std::string name;
        std::string value;
      public:
        void setName(std::string n);
        std::string getName();
        void setValue(std::string v);
        std::string getValue();
    };
    
    typedef std::vector<CxmlNode*> childlist;
    typedef std::vector<CxmlAttribute*> attributelist;
    
    std::string name;
    std::string text;
    CxmlNode* parent;
    childlist      children;
    attributelist  attributes;
    
    void p_printTree(std::string levelname);
    void p_writeToFile(FILE* xmlfile, std::string prefix);
  
  public:
    CxmlNode(std::string n);        // Konstruktor
    
    // for Parser
    void         addText                  ( std::string t );                  // Hngt Text (zwischen Tags) an
    void         setAttribute             ( std::string n, std::string v );   // Fgt ein Attribut hinzu
    CxmlNode*    setChild                 ( std::string n );                  // Fgt ein Kindknoten ein
    
    // for Parser and User
    CxmlNode*    getParent                ( );                                // Liefert den Vaterknoten
    bool         isRoot                   ( );                                // Prft, ob der Knoten die Wurzel ist
    
    // for User
    std::string  getName                  ( );                                // Liefert Namen des Knotens
    std::string  getText                  ( );                                // Liefert den Text des Knotens
    
    // Attributes
    std::string  getAttribute             ( std::string name, int number=1 ); // Liefert den Wert des <number>-ten Attributs mit dem Namen <name>
    int          getAttributeNumberByName ( std::string name );               // Liefert die Zahl der Attribute, die den Namen <name> tragen
    std::string  getAttributeName         ( int i );                          // Liefert den Namen des <i>-ten Attribus
    std::string  getAttribute             ( int i );                          // Liefert den Wert des <i>-ten Attributs
    int          getAttributeNumber       ( );                                // Liefer die Zahl der Attribute
    
    // Children
    CxmlNode*    getChild                 ( std::string name, int number=1 ); // Liefert den <number>-ten Kindnkoten mit dem Namen <name>
    int          getChildNumberByName     ( std::string name );               // Liefert die Zahl der Kindknoten, die den Namen <name> tragen
    CxmlNode*    getChild                 ( int i );                          // Liefert den <i>-ten Kindknoten
    int          getChildNumber           ( );                                // Liefert die Zahl der Kindknoten
    CxmlNode*    findChildByValue         ( std::string name, std::string attr, std::string value );  // Liefert Kindknoten, dessen Attribut <attr> den Wert <value> hat
    
    // Manipulation
    /*void         deleteAttribute          ( int i );                     // Lscht das <i>-te Attribut
    void         deleteAttribute          ( std::string name );          // Lscht das erste Attribut mit dem Namen <name>
    void         deleteAttributes         ( std::string name, int i=-1); // Lscht alle (oder die ersten <i>) Attribute mit Namen <name>
    void         deleteAllAttributes      ( );                           // Lscht alle Attribute
    
    void         deleteNode               ( bool deleteChildren=true );  // Lscht den Knoten und hngt Kindknoten ggf. an den Vaterknoten an
    
    void         deleteChild              ( int i );                                              // Lscht den <i>-ten Kindknoten
    void         deleteChild              ( std:string name, bool deleteChildren=true );          // Lscht den ersten Kindknoten mit Namen <name>
    void         deleteChildren           ( std::string name, int=-1, bool deleteChildren=true ); // Lscht alle (oder die ersten <i>) Kindknoten mit dem Namen <name>
    void         deleteAllChildren        ( bool deleteChildren=true ); // Lscht alle Kindknoten/**/
    
    // Output
    void         writeToFile               ( std::string filename );           // schreibt eine XML Datei raus
    
    // for debug
    void         printTree                ( );                                // Gibt den augespannten Baum im Detail aus
};

#endif